@@ -24,10 +24,7 @@
 
 /*****************************************************************************/
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -35,20 +32,22 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <byteswap.h>
 #include <usb.h>
 
-/* NOTE:  that should be <libusb.h> and it should include
- * <linux/usb_ch9.h> ... without it, we keep accumulating
- * potentially broken variants of standard types ...
- */
-
 #include "names.h"
 #include "devtree.h"
 #include "usbmisc.h"
 
-#define _GNU_SOURCE
 #include <getopt.h>
 
+#if (__BYTE_ORDER == __LITTLE_ENDIAN)
+  #define le16_to_cpu(x) (x)
+#elif (__BYTE_ORDER == __BIG_ENDIAN)
+  #define le16_to_cpu(x) bswap_16(x)
+#else
+  #error missing BYTE_ORDER
+#endif
 
 /* from USB 2.0 spec and updates */
 #define USB_DT_DEVICE_QUALIFIER		0x06
@@ -56,15 +55,36 @@
 #define USB_DT_OTG			0x09
 #define USB_DT_DEBUG			0x0a
 #define USB_DT_INTERFACE_ASSOCIATION	0x0b
+#define USB_DT_SECURITY			0x0c
+#define USB_DT_KEY			0x0d
+#define USB_DT_ENCRYPTION_TYPE		0x0e
+#define USB_DT_BOS			0x0f
+#define USB_DT_DEVICE_CAPABILITY	0x10
+#define USB_DT_WIRELESS_ENDPOINT_COMP	0x11
+#define USB_DT_WIRE_ADAPTER		0x21
+#define USB_DT_RPIPE			0x22
+
+#define USB_DT_RC_INTERFACE		0x23
+
+/* Conventional codes for class-specific descriptors.  The convention is
+ * defined in the USB "Common Class" Spec (3.11).  Individual class specs
+ * are authoritative for their usage, not the "common class" writeup.
+ */
+#define USB_DT_CS_DEVICE		(USB_TYPE_CLASS | USB_DT_DEVICE)
+#define USB_DT_CS_CONFIG		(USB_TYPE_CLASS | USB_DT_CONFIG)
+#define USB_DT_CS_STRING		(USB_TYPE_CLASS | USB_DT_STRING)
+#define USB_DT_CS_INTERFACE		(USB_TYPE_CLASS | USB_DT_INTERFACE)
+#define USB_DT_CS_ENDPOINT		(USB_TYPE_CLASS | USB_DT_ENDPOINT)
 
-/* convention suggested by common class spec */
-#define USB_DT_CS_INTERFACE		0x24
-#define USB_DT_CS_ENDPOINT		0x25
 
 #ifndef USB_CLASS_VIDEO
 #define USB_CLASS_VIDEO			0x0e
 #endif
 
+#ifndef USB_CLASS_APPLICATION
+#define USB_CLASS_APPLICATION	       	0xfe
+#endif
+
 #define VERBLEVEL_DEFAULT 0	/* 0 gives lspci behaviour; 1, lsusb-0.9 */
 
 #define CTRL_RETRIES	 2
@@ -72,9 +92,11 @@
 
 #define	HUB_STATUS_BYTELEN	3	/* max 3 bytes status = hub + 23 ports */
 
+extern int lsusb_t(void);
 static const char *procbususb = "/proc/bus/usb";
 static unsigned int verblevel = VERBLEVEL_DEFAULT;
 static int do_report_desc = 1;
+static const char *encryption_type[] = {"UNSECURE", "WIRED", "CCM_1", "RSA_1", "RESERVED"};
 
 static void dump_interface(struct usb_dev_handle *dev, struct usb_interface *interface);
 static void dump_endpoint(struct usb_dev_handle *dev, struct usb_interface_descriptor *interface, struct usb_endpoint_descriptor *endpoint);
@@ -83,6 +105,7 @@ static void dump_audiostreaming_interface(unsigned char *buf);
 static void dump_midistreaming_interface(struct usb_dev_handle *dev, unsigned char *buf);
 static void dump_videocontrol_interface(struct usb_dev_handle *dev, unsigned char *buf);
 static void dump_videostreaming_interface(unsigned char *buf);
+static void dump_dfu_interface(unsigned char *buf);
 static char *dump_comm_descriptor(struct usb_dev_handle *dev, unsigned char *buf, char *indent);
 static void dump_hid_device(struct usb_dev_handle *dev, struct usb_interface_descriptor *interface, unsigned char *buf);
 static void dump_audiostreaming_endpoint(unsigned char *buf);
@@ -137,6 +160,11 @@ static int get_string(struct usb_dev_handle *dev, char* buf, size_t size, u_int8
 {
 	int ret;
 
+	if (!dev) {
+		buf[0] = 0;
+		return 0;
+	}
+
 	if (id) {
 		ret = usb_get_string_simple(dev, id, buf, size);
 		if (ret <= 0) {
@@ -240,10 +268,21 @@ static int get_videoterminal_string(char *buf, size_t size, u_int16_t termt)
 static const char *get_guid(unsigned char *buf)
 {
 	static char guid[39];
-	sprintf(guid, "{%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x}",
-	       buf[3], buf[2], buf[1], buf[0], buf[5], buf[4], buf[7], buf[6],
-	       buf[8], buf[9], buf[10], buf[11], buf[12], buf[13], buf[14],
-	       buf[15]);
+
+	/* NOTE:  see RFC 4122 for more information about GUID/UUID
+	 * structure.  The first fields fields are historically big
+	 * endian numbers, dating from Apollo mc68000 workstations.
+	 */
+	sprintf(guid, "{%02x%02x%02x%02x"
+			"-%02x%02x"
+			"-%02x%02x"
+			"-%02x%02x"
+			"-%02x%02x%02x%02x%02x%02x}",
+	       buf[0], buf[1], buf[2], buf[3],
+	       buf[4], buf[5],
+	       buf[6], buf[7],
+	       buf[8], buf[9],
+	       buf[10], buf[11], buf[12], buf[13], buf[14], buf[15]);
 	return guid;
 }
 
@@ -323,6 +362,65 @@ static void dump_device(
 	       descriptor->bNumConfigurations);
 }
 
+static void dump_wire_adapter(unsigned char *buf)
+{
+
+	printf( "      Wire Adapter Class Descriptor:\n"
+		"        bLength             %5u\n"
+		"        bDescriptorType     %5u\n"
+		"        bcdWAVersion        %2x.%02x\n"
+		"	 bNumPorts	     %5u\n"
+		"	 bmAttributes	     %5u\n"
+		"	 wNumRPRipes	     %5u\n"
+		"	 wRPipeMaxBlock	     %5u\n"
+		"	 bRPipeBlockSize     %5u\n"
+		"	 bPwrOn2PwrGood	     %5u\n"
+		"	 bNumMMCIEs	     %5u\n"
+		"	 DeviceRemovable     %5u\n",
+		buf[0], buf[1], buf[3],
+		buf[2], buf[4], buf[5],
+		( buf[6] | buf[7] << 8 ),
+		( buf[8] | buf[9] << 8 ),
+		buf[10], buf[11], buf[12], buf[13]);
+}
+
+static void dump_rc_interface(unsigned char *buf)
+{
+	printf( "      Radio Control Interface Class Descriptor:\n"
+		"        bLength             %5u\n"
+		"        bDescriptorType     %5u\n"
+		"        bcdRCIVersion       %2x.%02x\n",
+		buf[0], buf[1], buf[3], buf[2]);
+
+}
+
+static void dump_security(unsigned char *buf)
+{
+	printf( "    Security Descriptor:\n"
+		"      bLength             %5u\n"
+		"      bDescriptorType     %5u\n"
+		"      wTotalLength        %5u\n"
+		"      bNumEncryptionTypes %5u\n",
+		buf[0], buf[1],
+		(buf[3]<< 8 | buf[2]),
+		buf[4]);
+}
+
+static void dump_encryption_type(unsigned char *buf)
+{
+	int b_encryption_type = buf[2] & 0x4;
+
+	printf( "    Encryption Type Descriptor:\n"
+		"      bLength             %5u\n"
+		"      bDescriptorType     %5u\n"
+		"      bEncryptionType     %5u %s\n"
+		"      bEncryptionValue    %5u\n"
+		"      bAuthKeyIndex       %5u\n",
+		buf[0], buf[1],
+		buf[2], encryption_type[b_encryption_type],
+		buf[3], buf[4]);
+}
+
 static void dump_association(struct usb_dev_handle *dev, unsigned char *buf)
 {
 	char cls[128], subcls[128], proto[128];
@@ -363,8 +461,10 @@ static void dump_config(struct usb_dev_handle *dev, struct usb_config_descriptor
 	       "    bConfigurationValue %5u\n"
 	       "    iConfiguration      %5u %s\n"
 	       "    bmAttributes         0x%02x\n",
-	       config->bLength, config->bDescriptorType, config->wTotalLength,
-	       config->bNumInterfaces, config->bConfigurationValue, config->iConfiguration,
+	       config->bLength, config->bDescriptorType,
+	       le16_to_cpu(config->wTotalLength),
+	       config->bNumInterfaces, config->bConfigurationValue,
+	       config->iConfiguration,
 	       cfg, config->bmAttributes);
 	if (!(config->bmAttributes & 0x80))
 		printf("      (Missing must-be-set bit!)\n");
@@ -395,9 +495,15 @@ static void dump_config(struct usb_dev_handle *dev, struct usb_config_descriptor
 			case USB_DT_INTERFACE_ASSOCIATION:
 				dump_association(dev, buf);
 				break;
+			case USB_DT_SECURITY:
+				dump_security(buf);
+				break;
+			case USB_DT_ENCRYPTION_TYPE:
+				dump_encryption_type(buf);
+				break;
 			default:
 				/* often a misplaced class descriptor */
-				printf("    UNRECOGNIZED: ");
+				printf("    ** UNRECOGNIZED: ");
 				dump_bytes(buf, buf[0]);
 				break;
 			}
@@ -447,63 +553,113 @@ static void dump_altsetting(struct usb_dev_handle *dev, struct usb_interface_des
 				dump_junk(buf, "      ", size);
 				break;
 			}
+
 			switch (buf[1]) {
+
+			/* This is the polite way to provide class specific
+			 * descriptors: explicitly tagged, using common class
+			 * spec conventions.
+			 */
+			case USB_DT_CS_DEVICE:
 			case USB_DT_CS_INTERFACE:
-				switch(interface->bInterfaceClass) {
-					case USB_CLASS_AUDIO:
-						switch(interface->bInterfaceSubClass) {
-							case 1:
-								dump_audiocontrol_interface(dev, buf);
-								break;
-							case 2:
-								dump_audiostreaming_interface(buf);
-								break;
-							case 3:
-								dump_midistreaming_interface(dev, buf);
-								break;
-						}
+				switch (interface->bInterfaceClass) {
+				case USB_CLASS_AUDIO:
+					switch (interface->bInterfaceSubClass) {
+					case 1:
+						dump_audiocontrol_interface(dev, buf);
 						break;
-					case USB_CLASS_COMM:
-						dump_comm_descriptor(dev, buf,
-							"      ");
+					case 2:
+						dump_audiostreaming_interface(buf);
 						break;
-					case USB_CLASS_VIDEO:
-						switch(interface->bInterfaceSubClass) {
-							case 1:
-								dump_videocontrol_interface(dev, buf);
-								break;
-							case 2:
-								dump_videostreaming_interface(buf);
-								break;
-						}
+					case 3:
+						dump_midistreaming_interface(dev, buf);
 						break;
 					default:
-						printf("      Class specific interface descriptor for " \
-							"class %u is unsupported\n", interface->bInterfaceClass);
+						goto dump;
+					}
+					break;
+				case USB_CLASS_COMM:
+					dump_comm_descriptor(dev, buf,
+						"      ");
+					break;
+				case USB_CLASS_VIDEO:
+					switch (interface->bInterfaceSubClass) {
+					case 1:
+						dump_videocontrol_interface(dev, buf);
+						break;
+					case 2:
+						dump_videostreaming_interface(buf);
+						break;
+					default:
+						goto dump;
+					}
+					break;
+				case USB_CLASS_APPLICATION:
+					switch (interface->bInterfaceSubClass) {
+					case 1:
+						dump_dfu_interface(buf);
+						break;
+					default:
+						goto dump;
+					}
+					break;
+				default:
+					goto dump;
 				}
 				break;
-			case USB_DT_HID:
-				if (interface->bInterfaceClass == USB_CLASS_HID)
-					dump_hid_device(dev, interface, buf);
-				if (interface->bInterfaceClass == 0x0b)
-					dump_ccid_device(buf);
-				break;
-			case USB_DT_OTG:
-				/* handled separately */
-				break;
-			case USB_DT_INTERFACE_ASSOCIATION:
-				dump_association(dev, buf);
-				break;
+
+			/* This is the ugly way:  implicitly tagged,
+			 * each class could redefine the type IDs.
+			 */
 			default:
-				/* often a misplaced class descriptor */
-				printf("      UNRECOGNIZED: ");
-				dump_bytes(buf, buf[0]);
-				break;
+				switch (interface->bInterfaceClass) {
+				case USB_CLASS_HID:
+					dump_hid_device(dev, interface, buf);
+					break;
+				case 0x0b:	/* chip/smartcard */
+					dump_ccid_device(buf);
+					break;
+				case 0xe0:	/* wireless */
+					switch (interface->bInterfaceSubClass) {
+					case 1:
+						switch (interface->bInterfaceProtocol) {
+						case 2:
+							dump_rc_interface(buf);
+							break;
+						default:
+							goto dump;
+						}
+						break;
+					case 2:
+						dump_wire_adapter(buf);
+						break;
+					default:
+						goto dump;
+					}
+					break;
+				default:
+					/* ... not everything is class-specific */
+					switch (buf[1]) {
+					case USB_DT_OTG:
+						/* handled separately */
+						break;
+					case USB_DT_INTERFACE_ASSOCIATION:
+						dump_association(dev, buf);
+						break;
+					default:
+dump:
+						/* often a misplaced class descriptor */
+						printf("      ** UNRECOGNIZED: ");
+						dump_bytes(buf, buf[0]);
+						break;
+					}
+				}
 			}
 			size -= buf[0];
 			buf += buf[0];
 		}
 	}
+
 	for (i = 0 ; i < interface->bNumEndpoints ; i++)
 		dump_endpoint(dev, interface, &interface->endpoint[i]);
 }
@@ -524,6 +680,7 @@ static void dump_endpoint(struct usb_dev_handle *dev, struct usb_interface_descr
 	static const char *hb[] = { "1x", "2x", "3x", "(?\?)" };
 	unsigned char *buf;
 	unsigned size;
+	unsigned wmax = le16_to_cpu(endpoint->wMaxPacketSize);
 
 	printf("      Endpoint Descriptor:\n"
 	       "        bLength             %5u\n"
@@ -538,9 +695,8 @@ static void dump_endpoint(struct usb_dev_handle *dev, struct usb_interface_descr
 	       endpoint->bLength, endpoint->bDescriptorType, endpoint->bEndpointAddress, endpoint->bEndpointAddress & 0x0f,
 	       (endpoint->bEndpointAddress & 0x80) ? "IN" : "OUT", endpoint->bmAttributes,
 	       typeattr[endpoint->bmAttributes & 3], syncattr[(endpoint->bmAttributes >> 2) & 3],
-	       usage[(endpoint->bmAttributes >> 4) & 3], endpoint->wMaxPacketSize,
-	       hb[(endpoint->wMaxPacketSize >> 11) & 3],
-	       endpoint->wMaxPacketSize & 0x3ff,
+	       usage[(endpoint->bmAttributes >> 4) & 3],
+	       wmax, hb[(wmax >> 11) & 3], wmax & 0x7ff,
 	       endpoint->bInterval);
 	/* only for audio endpoints */
 	if (endpoint->bLength == 9)
@@ -587,7 +743,7 @@ static void dump_endpoint(struct usb_dev_handle *dev, struct usb_interface_descr
 				break;
 			default:
 				/* often a misplaced class descriptor */
-				printf("        UNRECOGNIZED: ");
+				printf("        ** UNRECOGNIZED: ");
 				dump_bytes(buf, buf[0]);
 				break;
 			}
@@ -1497,7 +1653,7 @@ static void dump_videostreaming_interface(unsigned char *buf)
 		n = buf[12];
 		if (buf[0] < 13+p*n)
 			printf("      Warning: Descriptor too short\n");
-		printf("        bNumFormarts                    %5u\n"
+		printf("        bNumFormats                     %5u\n"
 		       "        wTotalLength                    %5u\n"
 		       "        bEndPointAddress                %5u\n"
 		       "        bmInfo                          %5u\n"
@@ -1506,10 +1662,12 @@ static void dump_videostreaming_interface(unsigned char *buf)
 		       "        bTriggerSupport                 %5u\n"
 		       "        bTriggerUsage                   %5u\n"
 		       "        bControlSize                    %5u\n",
-		       p, buf[5] | (buf[6] << 8), buf[7], buf[8], buf[9],
-		       buf[10], buf[11], buf[12], n);
+		       p, buf[4] | (buf[5] << 8), buf[6], buf[7], buf[8],
+		       buf[9], buf[10], buf[11], n);
 		for(i = 0; i < p; i++)
-			printf("        bmaControls(%2u)                 %5u\n", i, buf[13+p*n]);
+			printf(
+			"        bmaControls(%2u)                 %5u\n",
+				i, buf[13+p*n]);
 		dump_junk(buf, "        ", 13+p*n);
 		break;
 
@@ -1524,9 +1682,11 @@ static void dump_videostreaming_interface(unsigned char *buf)
 		       "        bEndpointAddress            %5u\n"
 		       "        bTerminalLink               %5u\n"
 		       "        bControlSize                %5u\n",
-		       p, buf[4] | (buf[5] << 8), buf[6], buf[8], n);
+		       p, buf[4] | (buf[5] << 8), buf[6], buf[7], n);
 		for(i = 0; i < p; i++)
-			printf("        bmaControls(%2u)             %5u\n", i, buf[9+p*n]);
+			printf(
+			"        bmaControls(%2u)             %5u\n",
+				i, buf[9+p*n]);
 		dump_junk(buf, "        ", 9+p*n);
 		break;
 
@@ -1700,6 +1860,34 @@ static void dump_videostreaming_interface(unsigned char *buf)
 	}
 }
 
+static void dump_dfu_interface(unsigned char *buf)
+{
+	if (buf[1] != USB_DT_CS_DEVICE)
+		printf("      Warning: Invalid descriptor\n");
+	else if (buf[0] < 7)
+		printf("      Warning: Descriptor too short\n");
+	printf("      Device Firmware Upgrade Interface Descriptor:\n"
+	       "        bLength                         %5u\n"
+	       "        bDescriptorType                 %5u\n"
+	       "        bmAttributes                    %5u\n",
+	       buf[0], buf[1], buf[2]);
+	if (buf[2] & 0xf0)
+		printf("          (unknown attributes!)\n");
+	printf("          Will %sDetach\n", (buf[2] & 0x08) ? "" : "Not ");
+	printf("          Manifestation %s\n", (buf[2] & 0x04) ? "Tolerant" : "Intolerant");
+	printf("          Upload %s\n", (buf[2] & 0x02) ? "Supported" : "Unsupported");
+	printf("          Download %s\n", (buf[2] & 0x01) ? "Supported" : "Unsupported");
+	printf("        wDetachTimeout                  %5u milliseconds\n"
+	       "        wTransferSize                   %5u bytes\n",
+	       buf[3] | (buf[4] << 8), buf[5] | (buf[6] << 8));
+
+	/* DFU 1.0 defines no version code, DFU 1.1 does */
+	if (buf[0] < 9)
+		return;
+	printf("        bcdDFUVersion                   %x.%02x\n",
+			buf[7], buf[8]);
+}
+
 static void dump_hub(char *prefix, unsigned char *p, int has_tt)
 {
 	unsigned int l, i, j;
@@ -2040,6 +2228,12 @@ static void dump_hid_device(struct usb_dev_handle *dev, struct usb_interface_des
 	if (!do_report_desc)
 		return;
 
+	if (!dev) {
+		printf( "         Report Descriptors: \n"
+			"           ** UNAVAILABLE **\n");
+		return;
+	}
+
 	for (i = 0; i < buf[5]; i++) {
 		/* we are just interested in report descriptors*/
 		if (buf[6+3*i] != USB_DT_REPORT)
@@ -2165,7 +2359,22 @@ dump_comm_descriptor(struct usb_dev_handle *dev, unsigned char *buf, char *inden
 			printf("%s    simple mode\n", indent);
 		break;
 	// case 0x09:		/* USB terminal */
-	// case 0x0a:		/* network channel terminal */
+	case 0x0a:		/* network channel terminal */
+		type = "Network Channel Terminal";
+		if (buf [0] != 7)
+			goto bad;
+		get_string(dev, str, sizeof str, buf[4]);
+		printf("%sNetwork Channel Terminal:\n"
+		       "%s  bEntityId               %3d\n"
+		       "%s  iName                   %3d %s\n"
+		       "%s  bChannelIndex           %3d\n"
+		       "%s  bPhysicalInterface      %3d\n",
+		       indent,
+		       indent, buf[3],
+		       indent, buf[4], str,
+		       indent, buf[5],
+		       indent, buf[6]);
+		break;
 	// case 0x0b:		/* protocol unit */
 	// case 0x0c:		/* extension unit */
 	// case 0x0d:		/* multi-channel management */
@@ -2593,6 +2802,8 @@ dump_device_status(struct usb_dev_handle *fd, int otg, int wireless)
 static int do_wireless(struct usb_dev_handle *fd)
 {
 	/* FIXME fetch and dump BOS etc */
+	if (fd)
+		return 0;
 	return 0;
 }
 
@@ -2604,28 +2815,31 @@ static void dumpdev(struct usb_device *dev)
 
 	otg = wireless = 0;
 	udev = usb_open(dev);
-	if (udev) {
-		dump_device(udev, &dev->descriptor);
-		if (dev->descriptor.bcdUSB >= 0x0250)
-			wireless = do_wireless(udev);
-		if (dev->config) {
-			otg = do_otg(&dev->config[0]) || otg;
-			for (i = 0; i < dev->descriptor.bNumConfigurations;
-					i++) {
-				dump_config(udev, &dev->config[i]);
-			}
+	if (!udev)
+		fprintf(stderr, "Couldn't open device, some information "
+			"will be missing\n");
+
+	dump_device(udev, &dev->descriptor);
+	if (dev->descriptor.bcdUSB >= 0x0250)
+		wireless = do_wireless(udev);
+	if (dev->config) {
+		otg = do_otg(&dev->config[0]) || otg;
+		for (i = 0; i < dev->descriptor.bNumConfigurations;
+				i++) {
+			dump_config(udev, &dev->config[i]);
 		}
-		if (dev->descriptor.bDeviceClass == USB_CLASS_HUB)
-			do_hub(udev, dev->descriptor.bDeviceProtocol);
-		if (dev->descriptor.bcdUSB >= 0x0200) {
-			do_dualspeed(udev);
-			do_debug(udev);
-		}
-		dump_device_status(udev, otg, wireless);
-		usb_close(udev);
 	}
-	else
-		fprintf(stderr, "Couldn't open device\n");
+	if (!udev)
+		return;
+
+	if (dev->descriptor.bDeviceClass == USB_CLASS_HUB)
+		do_hub(udev, dev->descriptor.bDeviceProtocol);
+	if (dev->descriptor.bcdUSB >= 0x0200) {
+		do_dualspeed(udev);
+		do_debug(udev);
+	}
+	dump_device_status(udev, otg, wireless);
+	usb_close(udev);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -2719,6 +2933,8 @@ static int treedump(void)
 	char buf[512];
 
 	snprintf(buf, sizeof(buf), "%s/devices", procbususb);
+	if (access(buf, R_OK) < 0)
+		return lsusb_t();
 	if ((fd = open(buf, O_RDONLY)) == -1) {
 		fprintf(stderr, "cannot open %s, %s (%d)\n", buf, strerror(errno), errno);
 		return(1);
@@ -2769,12 +2985,12 @@ int main(int argc, char *argv[])
 			if (cp) {
 				*cp++ = 0;
 				if (*optarg)
-					bus = strtoul(optarg, NULL, 0);
+					bus = strtoul(optarg, NULL, 10);
 				if (*cp)
-					devnum = strtoul(cp, NULL, 0);
+					devnum = strtoul(cp, NULL, 10);
 			} else {
 				if (*optarg)
-					devnum = strtoul(optarg, NULL, 0);
+					devnum = strtoul(optarg, NULL, 10);
 			}
 			break;
 
@@ -2823,14 +3039,16 @@ int main(int argc, char *argv[])
 	}
 
 	/* by default, print names as well as numbers */
-	if ((err = names_init("./usb.ids")) != 0) {
-		if ((err = names_init(USBIDS_FILE)) != 0) {
-			fprintf(stderr, "%s: cannot open \"%s\", %s\n",
-					argv[0],
-					USBIDS_FILE,
-					strerror(err));
-		}
-	}
+	err = names_init(DATADIR "/usb.ids");
+#ifdef HAVE_LIBZ
+	if (err != 0)
+		err = names_init(DATADIR "/usb.ids.gz");
+#endif
+	if (err != 0)
+		fprintf(stderr, "%s: cannot open \"%s\", %s\n",
+				argv[0],
+				DATADIR "/usb.ids",
+				strerror(err));
 	status = 0;
 
 	usb_init();
@@ -2838,7 +3056,6 @@ int main(int argc, char *argv[])
 	usb_find_busses();
 	usb_find_devices();
 
-
 	if (treemode) {
 		/* treemode requires at least verblevel 1 */
  		verblevel += 1 - VERBLEVEL_DEFAULT;
